#!/bin/bash

echo ""
FILE="3.1.dtd"

function die {
  echo "ERROR: $@"
  exit 1
}

function check_file {
  [[ -f "$1" ]] || die "FILE '$1' not found"
}

function strip {
  s=$(cat)
  s=${s## }
  echo ${s%% }
}

check_file "$FILE"

xmllint --version 2> /dev/null || die "xmllint not found. Please install it."
xsltproc --version > /dev/null || die "xsltproc not found. Please install it."

check_file "3.1.xml"
xmllint --noout 3.1.xml || die "File '3.1.xml' is not a valid XML file. Did you modify it?"

check_file "3.1.xml"
xmllint --noout --dtdvalid "$FILE" 3.1.xml\
  || die "DTD file '$FILE' is not a valid DTD, or file '3.1.xml' does ot validate with it."
echo "Test 1 PASSED"
echo ""
echo -n "Running Test 2... "
failures=""
failed=0
tests=0
for i in 3.1.test*.xml; do
  (( tests = tests + 1 ))
  if xmllint --noout --dtdvalid "$FILE" "$i" 2> /dev/null; then
    failures+="$i "
    (( failed = failed + 1))
  fi
done
if [[ -z "${failures}" ]]; then
  echo "Test 2 PASSED PERFECTLY (on $tests tests)! Your DTD is as strict as it can be!!!"
  exit 0
fi
if (( failed > 20 )); then
  echo "Your DTD must be a lot stricter!"
elif (( failed > 10 )); then
  echo "Your DTD has some strictness, but you should really try to make it stricter."
elif (( failed > 5 )); then
  echo "Your DTD is somewhat strict. But it could be better."
elif (( failed > 2 )); then
  echo "Your DTD is quite strict. Not perfect yet!"
else
  echo "Your DTD is almost perfect!"
fi
echo "Your DTD (wrongly!) validated $failed of $tests 'intentionally' bad files.\
 Ideally it would have validated 0 of them."
echo "You can inspect the corresponding test files: inside, there is a comment explaining why the DTD should *not* validate it."
echo ""
echo "List of bad files validated by your DTD: $failures"

exit 1
